#!/usr/bin/env python

# Ubuntu Testing Automation Harness
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.

# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

import argparse
import jenkins
import jinja2
import logging
import os

from bzrlib.branch import Branch


def _job(device, adb_serial, ncd_bank, ncd_serial):
    return {
        'device': device.split('-', 2)[0],  # turn grouper-01 into grouper
        'device_name': device,
        'adb_serial': adb_serial,
        'ncd_bank': ncd_bank,
        'ncd_serial': ncd_serial,
    }


# this defines all the permutations we want to test
PERMUTATIONS = [
    _job('grouper-05', '015d21d9151c0019', 0, 0),
]


def _get_parser():
    """Create and return command line parser.

    :returns: command line parser
    :rtype: argparse.ArgumentParser

    """
    parser = argparse.ArgumentParser(
        description='Create/Update upgrade testing jenkins jobs.')
    parser.add_argument("-d", "--dryrun", action="store_true",
                        help="Dry run mode. Don't execute jenkins commands.")
    parser.add_argument("-u", "--username",
                        help="username to use when logging into Jenkins.")
    parser.add_argument("-p", "--password",
                        help="username to use when logging into Jenkins")
    parser.add_argument("-j", "--jenkins", default="http://10.98.0.1:8080/",
                        help="URL of jenkins instance to configure jobs in.")
    parser.add_argument("-n", "--node", default="phoenix",
                        help=("Jenkins node were the job should be executed "
                              "(default=%(default)s)"))
    return parser


def _get_jenkins(url, username, password):
    logging.info('Attempting to login to jenkins at %s', url)
    if username is not None:
        logging.info('...with authentication as user: %s', username)
        instance = jenkins.Jenkins(url, username=username, password=password)
    else:
        logging.info('...without authentication')
        instance = jenkins.Jenkins(url)

    return instance


def _get_environment():
    base = os.path.dirname(__file__)
    return jinja2.Environment(
        loader=jinja2.FileSystemLoader(base),
        undefined=jinja2.StrictUndefined,
        trim_blocks=True)


def _get_job_name(job_args):
    fmt = 'power_software-saucy-touch-armhf-install-idle-{device}'
    return fmt.format(**job_args)


def _get_branch():
    path = os.path.dirname(__file__)
    path = os.path.join(path, '..')
    branch = Branch.open(path).get_parent()
    if branch.endswith('/'):
        branch = branch[:-1]
    return branch.replace('bzr+ssh://bazaar.launchpad.net/', 'lp:')


def _configure_job(instance, env, node, job_args, dryrun):
    tmpl = env.get_template('config.xml.jinja2')
    cfg = tmpl.render(node=node, branch=_get_branch(), **job_args)

    jobname = _get_job_name(job_args)
    if instance.job_exists(jobname):
        logging.info('reconfiguring job %s', jobname)
        instance.reconfig_job(jobname, cfg)
    else:
        logging.info('creating job %s', jobname)
        instance.create_job(jobname, cfg)


def _dryrun_func(jobname, config):
    logging.debug(config)


def main():
    logging.basicConfig(level=logging.DEBUG)
    args = _get_parser().parse_args()

    jenkins_inst = _get_jenkins(args.jenkins, args.username, args.password)
    if args.dryrun:
        jenkins_inst.create_job = _dryrun_func
        jenkins_inst.reconfig_job = _dryrun_func

    env = _get_environment()

    for job_args in PERMUTATIONS:
        _configure_job(jenkins_inst, env, args.node, job_args, args.dryrun)

if __name__ == '__main__':
    main()
